package pobyty.business;

import java.util.List;
import org.smp.SmpException;
import pobyty.model.Datum;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.PobytyIntegrityException;
import pobyty.model.Poznamka;
import pobyty.model.PrihlaskaKlient;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.doprovod.Doprovod;
import pobyty.model.doprovod.DoprovodId;
import pobyty.model.klient.Indikace;
import pobyty.model.klient.Klient;
import pobyty.model.klient.KlientId;
import pobyty.model.klient.Kontraindikace;
import pobyty.model.prihlaska.Prihlaska;
import pobyty.model.prihlaska.StavPrihlasky;
import pobyty.model.skupina.Skupina;
import pobyty.model.skupina.SkupinaId;
import pobyty.model.skupina.VekKlientu;
import pobyty.model.turnus.Cena;
import pobyty.model.turnus.Kapacita;
import pobyty.model.turnus.Lokalita;
import pobyty.model.turnus.Turnus;
import pobyty.model.turnus.TurnusId;
import pobyty.model.turnus.Zkratka;
import pobyty.model.uzivatel.Heslo;
import pobyty.model.uzivatel.RoleUzivatele;
import pobyty.model.uzivatel.Username;
import pobyty.model.uzivatel.Uzivatel;

public abstract class Facade {

    private static Facade instance;

    /**
     * @return the instance
     */
    public static Facade getInstance() {
        if (instance == null) {
            instance = new FacadeImpl();
        }
        return instance;
    }
  
    /*
     * Uzivatel ***************************************************
     */
    public abstract void addUzivatel(Username username, Prijmeni prijmeni, Jmeno jmeno, RoleUzivatele roleUzivatele, Heslo heslo) throws SmpException;

    public abstract Uzivatel findUzivatel(Username username) throws SmpException;

    public abstract List<Uzivatel> allUzivatel() throws SmpException;

    public abstract void zmenitHeslo(Username username, Heslo heslo) throws SmpException;

    public abstract void deleteUzivatel(Username username) throws SmpException;

    /*
     * Skupina ***************************************************
     */
    public abstract void addSkupina(TurnusId turnusId, VekKlientu vekKlientu, DoprovodId doprovodId) throws SmpException;

    public abstract Skupina findSkupina(SkupinaId skupinaId) throws SmpException;

    public abstract void deleteSkupina(SkupinaId skupinaId) throws SmpException, PobytyIntegrityException;

    public abstract void updateSkupina(SkupinaId skupinaId, DoprovodId doprovodId) throws SmpException;

    public abstract List<PrihlaskaKlient> klientiSkupiny(SkupinaId skupinaId) throws SmpException;

    public abstract List<Skupina> getSkupinyTurnusu(TurnusId turnusId) throws SmpException;

    public abstract List<Klient> kandidatiSkupiny(Skupina skupina) throws SmpException;

    /*
     * Prihlasky ***************************************************
     */
    public abstract void addPrihlaska(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException;

    public abstract Prihlaska findPrihlaska(KlientId id) throws SmpException;

    public abstract List<Prihlaska> prihlasky() throws SmpException;

    public abstract void updatePrihlaska(KlientId klientId, StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException;

    public abstract void updatePrihlaska(KlientId klientId, TurnusId turnusId, StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException;

    public abstract void updatePrihlaska(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException;

    public abstract void updatePrihlaska(KlientId KlientId, SkupinaId skupinaId) throws SmpException;

    public abstract void deletePrihlaska(KlientId klientId) throws SmpException;

    public abstract int pocetKlientuSkupiny(SkupinaId skupinaId) throws SmpException;

    /*
     * Klient ***************************************************
     */
    public abstract void addKlient(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno,
            Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto,
            Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException;

    public abstract Klient findKlient(KlientId id) throws SmpException;

    public abstract List<Klient> klienti() throws SmpException;

    public abstract List<Klient> klienti(String filtrPrijmeni, String filtrJmeno) throws SmpException;

    public abstract void updateKlient(KlientId klientId, Mejl mejl, Ulice ulice,
            Psc psc, Mesto mesto, Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException;

    public abstract void updateKlient(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno,
            Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon,
            Indikace indikace, Kontraindikace kontraindikace) throws SmpException;

    public abstract void deleteKlient(KlientId klientId) throws SmpException, PobytyIntegrityException;

    /*
     * Turnus ***************************************************
     */
    public abstract void addTurnus(Datum zacatek, Datum konec, Lokalita lokalita, Cena cena, Kapacita kapacita, Zkratka zkratka) throws SmpException;

    public abstract Turnus findTurnus(TurnusId turnusId) throws SmpException;

    public abstract List<Turnus> turnusy() throws SmpException;

    public abstract void updateTurnus(TurnusId turnusId, Zkratka zkratka, Datum zacatek, Datum konec, Lokalita lokalita, Cena cena, Kapacita kapacita) throws SmpException;

    public abstract void deleteTurnus(TurnusId turnusId) throws SmpException, PobytyIntegrityException;

    /*
     * Doprovod ***************************************************
     */
    public abstract void addDoprovod(Prijmeni prijmeni, Jmeno jmeno, Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon) throws SmpException;

    public abstract Doprovod findDoprovod(DoprovodId doprovodId) throws SmpException;

    public abstract List<Doprovod> doprovody() throws SmpException;

   // public abstract List<Doprovod> doprovody(String filtrPrijmeni) throws SmpException;

    public abstract List<Doprovod> doprovody(String filtrPrijmeni, String filtrJmeno) throws SmpException;

    public abstract void updateDoprovod(DoprovodId doprovodId, Prijmeni prijmeni, Jmeno jmeno, Mejl mejl, Ulice ulice, Mesto mesto, Psc psc, Telefon telefon) throws SmpException;

    public abstract void deleteDoprovod(DoprovodId doprovodId) throws SmpException;

    /*
     * Ostatni ***************************************************
     */
    public abstract List<PrihlaskaKlient> prihlaskaKlientJoin() throws SmpException;

    public abstract List<PrihlaskaKlient> prihlaskaKlientJoin(String filtrPrijmeni, String filtrJmeno) throws SmpException;

    public abstract List<PrihlaskaKlient> prihlaskaKlientJoin(StavPrihlasky stavPrihlasky) throws SmpException;

    public abstract List<PrihlaskaKlient> prihlaskaKlientJoin(TurnusId turnusId) throws SmpException;

    public abstract int pocetPrihlasenychNaTurnus(TurnusId turnusId) throws SmpException;
}
