package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLIntegrityConstraintViolationException;
import java.util.ArrayList;
import java.util.List;
import org.smp.SmpException;
import pobyty.business.KlientDAO;
import pobyty.model.Datum;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.OsobaAdresa;
import pobyty.model.PobytyIntegrityException;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.klient.Indikace;
import pobyty.model.klient.Klient;
import pobyty.model.klient.KlientId;
import pobyty.model.klient.Kontraindikace;

public class KlientDAODerby implements KlientDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updatePs;
    private final PreparedStatement updateAllPs;

    public KlientDAODerby(Connection connection) throws SQLException {
        createPs = connection.prepareStatement("INSERT INTO KLIENT VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
        findPs = connection.prepareStatement(
                String.format("SELECT * FROM KLIENT WHERE %s = ?", KlientId.colName()));
        findAllPs = connection.prepareStatement("SELECT * FROM KLIENT");
        updatePs = connection.prepareStatement(
                String.format("UPDATE KLIENT SET %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ? WHERE %s = ?",
                Mejl.colName(),
                Ulice.colName(),
                Mesto.colName(),
                Psc.colName(),
                Telefon.colName(),
                Indikace.colName(),
                Kontraindikace.colName(),
                KlientId.colName()));
        updateAllPs = connection.prepareStatement(
                String.format("UPDATE KLIENT SET %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ? WHERE %s = ?",
                Prijmeni.colName(),
                Jmeno.colName(),
                Klient.COLUMN_NAROZENI,
                Mejl.colName(),
                Ulice.colName(),
                Mesto.colName(),
                Psc.colName(),
                Telefon.colName(),
                Indikace.colName(),
                Kontraindikace.colName(),
                KlientId.colName()));
        deletePs = connection.prepareStatement(
                String.format("DELETE FROM KLIENT WHERE %s = ?", KlientId.colName()));
    }

    @Override
    public void add(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno, Datum narozeni,
            Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon, Indikace indikace,
            Kontraindikace kontraindikace) throws SmpException {
        int i = 0;
        try {
            createPs.setString(++i, klientId.getKlientId());
            createPs.setString(++i, prijmeni.getPrijmeni());
            createPs.setString(++i, jmeno.getJmeno());
            createPs.setDate(++i, narozeni.getDate());
            createPs.setString(++i, mejl.getMejl());
            createPs.setString(++i, ulice.getUlice());
            createPs.setString(++i, mesto.getMesto());
            createPs.setString(++i, psc.getPsc());
            createPs.setString(++i, telefon.getTelefon());
            createPs.setString(++i, indikace.getIndikaceId());
            createPs.setString(++i, kontraindikace.getKontraindikaceId());
            createPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private Klient loadKlient(ResultSet rs) throws SQLException {
        Klient klient = new Klient(new KlientId(rs.getString(KlientId.colName())),
                new Prijmeni(rs.getString(Prijmeni.colName())),
                new Jmeno(rs.getString(Jmeno.colName())),
                new Datum(rs.getDate(OsobaAdresa.COLUMN_NAROZENI)),
                new Mejl(rs.getString(Mejl.colName())),
                new Ulice(rs.getString(Ulice.colName())),
                new Mesto(rs.getString(Mesto.colName())),
                new Psc(rs.getString(Psc.colName())),
                new Telefon(rs.getString(Telefon.colName())),
                Indikace.valueOf(rs.getString(Indikace.colName())),
                Kontraindikace.valueOf(rs.getString(Kontraindikace.colName())));

        return klient;
    }

    @Override
    public Klient find(KlientId klientId) throws SmpException {
        try {
            findPs.setString(1, klientId.getKlientId());
            ResultSet rs = findPs.executeQuery();
            if (rs.next()) {
                return loadKlient(rs);
            }
            return null;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Klient> all() throws SmpException {
        try {
            List<Klient> list = new ArrayList<>();
            ResultSet rs = findAllPs.executeQuery();
            while (rs.next()) {
                list.add(loadKlient(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException {
        try {
            int i = 0;
            updatePs.setString(++i, mejl.getMejl());
            updatePs.setString(++i, ulice.getUlice());
            updatePs.setString(++i, mesto.getMesto());
            updatePs.setString(++i, psc.getPsc());
            updatePs.setString(++i, telefon.getTelefon());
            updatePs.setString(++i, indikace.getIndikaceId());
            updatePs.setString(++i, kontraindikace.getKontraindikaceId());
            updatePs.setString(++i, klientId.getKlientId());
            updatePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno, Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException {
        try {
            int i = 0;
            updateAllPs.setString(++i, prijmeni.getPrijmeni());
            updateAllPs.setString(++i, jmeno.getJmeno());
            updateAllPs.setDate(++i, narozeni.getDate());
            updateAllPs.setString(++i, mejl.getMejl());
            updateAllPs.setString(++i, ulice.getUlice());
            updateAllPs.setString(++i, mesto.getMesto());
            updateAllPs.setString(++i, psc.getPsc());
            updateAllPs.setString(++i, telefon.getTelefon());
            updateAllPs.setString(++i, indikace.getIndikaceId());
            updateAllPs.setString(++i, kontraindikace.getKontraindikaceId());
            updateAllPs.setString(++i, klientId.getKlientId());
            updateAllPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(KlientId doprovodId) throws SmpException, PobytyIntegrityException {
        try {
            deletePs.setString(1, doprovodId.getKlientId());
            deletePs.execute();
        } catch (SQLIntegrityConstraintViolationException ex) {
            throw new PobytyIntegrityException(ex);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }
}
