package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.smp.SmpException;
import pobyty.business.PrihlaskaDAO;
import pobyty.model.Datum;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.Poznamka;
import pobyty.model.PrihlaskaKlient;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.klient.Indikace;
import pobyty.model.klient.Klient;
import pobyty.model.klient.KlientId;
import pobyty.model.klient.Kontraindikace;
import pobyty.model.prihlaska.Prihlaska;
import pobyty.model.prihlaska.StavPrihlasky;
import pobyty.model.skupina.SkupinaId;
import pobyty.model.turnus.TurnusId;

public class PrihlaskaDAODerby implements PrihlaskaDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement findAllByGroupPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updateStavPoznPs;
    private final PreparedStatement updateTurnusPoznPs;
    private final PreparedStatement updateAllPs;
    private final PreparedStatement updateSkupinaPs;
    private final PreparedStatement countPs;
    private final PreparedStatement countTurnusPs;
    private final PreparedStatement prihlaskaKlientJoinPs;
    private final PreparedStatement prihlaskaKlientStavJoinPs;
    private final PreparedStatement prihlaskaKlientTurnusJoinPs;
    private final PreparedStatement prihlaskaKlientSkupinaJoinPs;

    public PrihlaskaDAODerby(Connection connection) throws SQLException {
        createPs = connection.prepareStatement("INSERT INTO PRIHLASKA VALUES(?, ?, DEFAULT, ?, ?)");
        findPs = connection.prepareStatement(
                String.format("SELECT * FROM PRIHLASKA WHERE %s = ?", KlientId.colName()));
        findAllPs = connection.prepareStatement("SELECT * FROM PRIHLASKA");
        findAllByGroupPs = connection.prepareStatement(
                String.format("SELECT * FROM PRIHLASKA WHERE %s = ?", SkupinaId.colName()));
        updateStavPoznPs = connection.prepareStatement(
                String.format("UPDATE PRIHLASKA SET %s = ?, %s = ? WHERE %s = ?",
                StavPrihlasky.colName(), Poznamka.colName(), KlientId.colName()));
        updateTurnusPoznPs = connection.prepareStatement(
                String.format("UPDATE PRIHLASKA SET %s = ?, %s = ? WHERE %s = ?",
                TurnusId.colName(), Poznamka.colName(), KlientId.colName()));
        updateAllPs = connection.prepareStatement(
                String.format("UPDATE PRIHLASKA SET %s = ?, %s = ?, %s = ? WHERE %s = ?",
                TurnusId.colName(), StavPrihlasky.colName(), Poznamka.colName(), KlientId.colName()));
        updateSkupinaPs = connection.prepareStatement(
                String.format("UPDATE PRIHLASKA SET %s = ? WHERE %s = ?",
                SkupinaId.colName(), KlientId.colName()));
        deletePs = connection.prepareStatement(
                String.format("DELETE FROM PRIHLASKA WHERE %s = ?", KlientId.colName()));
        countPs = connection.prepareStatement(
                String.format("SELECT COUNT(*) FROM PRIHLASKA WHERE %s = ?", SkupinaId.colName()));
        prihlaskaKlientJoinPs = connection.prepareStatement(
                String.format("SELECT * FROM KLIENT NATURAL JOIN PRIHLASKA"));
        prihlaskaKlientStavJoinPs = connection.prepareStatement(
                String.format("SELECT * FROM KLIENT NATURAL JOIN PRIHLASKA WHERE %s = ?", StavPrihlasky.colName()));
        prihlaskaKlientTurnusJoinPs = connection.prepareStatement(
                String.format("SELECT * FROM KLIENT NATURAL JOIN PRIHLASKA WHERE %s = ?", TurnusId.colName()));
        prihlaskaKlientSkupinaJoinPs = connection.prepareStatement(
                String.format("SELECT * FROM KLIENT NATURAL JOIN PRIHLASKA WHERE %s = ?", SkupinaId.colName()));
        countTurnusPs = connection.prepareStatement(
                String.format("SELECT COUNT(*) FROM PRIHLASKA WHERE %s = ?", TurnusId.colName()));

    }

    @Override
    public void add(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException {
        try {
            int i = 0;
            createPs.setString(++i, klientId.getKlientId());
            createPs.setInt(++i, turnusId.getAbstrtId());
            createPs.setString(++i, StavPrihlasky.PRIHLASEN.getStavPrihlaskyId());
            createPs.setString(++i, poznamka.toString());
            createPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private static Prihlaska loadPrihlaska(ResultSet rs) throws SQLException {
        return new Prihlaska(
                new KlientId(rs.getString(KlientId.colName())),
                new TurnusId(rs.getInt(TurnusId.colName())),
                new SkupinaId(rs.getInt(SkupinaId.colName())),
                StavPrihlasky.valueOf(rs.getString(StavPrihlasky.colName())),
                new Poznamka(rs.getString(Poznamka.colName())));
    }

    @Override
    public Prihlaska find(KlientId klientId) throws SmpException {
        try {
            findPs.setString(1, klientId.getKlientId());
            ResultSet rs = findPs.executeQuery();
            if (rs.next()) {
                return loadPrihlaska(rs);
            }
            return null;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Prihlaska> all() throws SmpException {
        try {
            ResultSet rs = findAllPs.executeQuery();
            List<Prihlaska> list = new ArrayList<>();
            while (rs.next()) {
                list.add(loadPrihlaska(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Prihlaska> all(SkupinaId skupinaId) throws SmpException {
        try {
            findAllByGroupPs.setInt(1, skupinaId.getAbstrtId());
            ResultSet rs = findAllByGroupPs.executeQuery();
            List<Prihlaska> list = new ArrayList<>();
            while (rs.next()) {
                list.add(loadPrihlaska(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException {
        try {
            updateStavPoznPs.setString(1, stavPrihlasky.getStavPrihlaskyId());
            updateStavPoznPs.setString(2, poznamka.getPoznamka());
            updateStavPoznPs.setString(3, klientId.getKlientId());
            updateStavPoznPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, TurnusId turnusId, StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException {
        try {
            int i = 0;
            updateAllPs.setInt(++i, turnusId.getAbstrtId());
            updateAllPs.setString(++i, stavPrihlasky.getStavPrihlaskyId());
            updateAllPs.setString(++i, poznamka.getPoznamka());
            updateAllPs.setString(++i, klientId.getKlientId());
            updateAllPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException {
        try {
            int i = 0;
            updateTurnusPoznPs.setInt(++i, turnusId.getAbstrtId());
            updateTurnusPoznPs.setString(++i, poznamka.getPoznamka());
            updateTurnusPoznPs.setString(++i, klientId.getKlientId());
            updateTurnusPoznPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(KlientId klientId, SkupinaId skupinaId) throws SmpException {
        try {
            if (skupinaId.equals(SkupinaId.NEDEF)) {
                updateSkupinaPs.setNull(1, Types.INTEGER);
            } else {
                updateSkupinaPs.setInt(1, skupinaId.getAbstrtId());
            }
            updateSkupinaPs.setString(2, klientId.getKlientId());
            updateSkupinaPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(KlientId klientId) throws SmpException {
        try {
            deletePs.setString(1, klientId.getKlientId());
            deletePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public int clientsCount(SkupinaId skupinaId) throws SmpException {
        try {
            countPs.setInt(1, skupinaId.getAbstrtId());
            ResultSet rs = countPs.executeQuery();
            rs.next();
            return rs.getInt(1);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public int clientsCount(TurnusId turnusId) throws SmpException {
        try {
            countTurnusPs.setInt(1, turnusId.getAbstrtId());
            ResultSet rs = countTurnusPs.executeQuery();
            rs.next();
            return rs.getInt(1);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private static PrihlaskaKlient loadPrihlaskaKlient(ResultSet rs) throws SQLException {
        PrihlaskaKlient pk = new PrihlaskaKlient(
                new Prijmeni(rs.getString(Prijmeni.colName())),
                new Jmeno(rs.getString(Jmeno.colName())),
                new Datum(rs.getDate(Klient.COLUMN_NAROZENI)),
                new Mejl(rs.getString(Mejl.colName())),
                new Ulice(rs.getString(Ulice.colName())),
                new Mesto(rs.getString(Mesto.colName())),
                new Psc(rs.getString(Psc.colName())),
                new Telefon(rs.getString(Telefon.colName())),
                Indikace.valueOf(rs.getString(Indikace.colName())),
                Kontraindikace.valueOf(rs.getString(Kontraindikace.colName())),
                new KlientId(rs.getString(KlientId.colName())),
                new TurnusId(rs.getInt(TurnusId.colName())),
                new SkupinaId(rs.getInt(SkupinaId.colName())),
                StavPrihlasky.valueOf(rs.getString(StavPrihlasky.colName())),
                new Poznamka(rs.getString(Poznamka.colName())));
        return pk;
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin() throws SmpException {
        List<PrihlaskaKlient> pkList = new ArrayList<>();
        try {
            ResultSet rs = prihlaskaKlientJoinPs.executeQuery();
            while (rs.next()) {
                pkList.add(loadPrihlaskaKlient(rs));
            }
            return pkList;
        } catch (SQLException ex) {
            Logger.getLogger(PrihlaskaDAODerby.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(StavPrihlasky stavPrihlasky) throws SmpException {
        List<PrihlaskaKlient> pkList = new ArrayList<>();
        try {
            prihlaskaKlientStavJoinPs.setString(1, stavPrihlasky.getStavPrihlaskyId());
            ResultSet rs = prihlaskaKlientStavJoinPs.executeQuery();
            while (rs.next()) {
                pkList.add(loadPrihlaskaKlient(rs));
            }
            return pkList;
        } catch (SQLException ex) {
            Logger.getLogger(PrihlaskaDAODerby.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(TurnusId turnusId) throws SmpException {
        List<PrihlaskaKlient> pkList = new ArrayList<>();
        try {
            prihlaskaKlientTurnusJoinPs.setInt(1, turnusId.getAbstrtId());
            ResultSet rs = prihlaskaKlientTurnusJoinPs.executeQuery();
            while (rs.next()) {
                pkList.add(loadPrihlaskaKlient(rs));
            }
            return pkList;
        } catch (SQLException ex) {
            Logger.getLogger(PrihlaskaDAODerby.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(SkupinaId skupinaId) throws SmpException {
        List<PrihlaskaKlient> pkList = new ArrayList<>();
        try {
            prihlaskaKlientSkupinaJoinPs.setInt(1, skupinaId.getAbstrtId());
            ResultSet rs = prihlaskaKlientSkupinaJoinPs.executeQuery();
            while (rs.next()) {
                pkList.add(loadPrihlaskaKlient(rs));
            }
            return pkList;
        } catch (SQLException ex) {
            Logger.getLogger(PrihlaskaDAODerby.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }
}
