package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.smp.SmpException;
import pobyty.business.UzivatelDAO;
import pobyty.model.Jmeno;
import pobyty.model.Prijmeni;
import pobyty.model.uzivatel.Heslo;
import pobyty.model.uzivatel.RoleUzivatele;
import pobyty.model.uzivatel.Username;
import pobyty.model.uzivatel.Uzivatel;

public class UzivatelDAODerby implements UzivatelDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updatePs;

    public UzivatelDAODerby(Connection connection) throws SQLException {
        deletePs = connection.prepareStatement("DELETE FROM UZIVATEL WHERE USERNAME = ?");
        createPs = connection.prepareStatement("INSERT INTO UZIVATEL VALUES(?, ?, ?, ?, ?)");
        findAllPs = connection.prepareStatement("SELECT * FROM UZIVATEL");
        findPs = connection.prepareStatement("SELECT * FROM UZIVATEL WHERE USERNAME = ?");
        updatePs = connection.prepareStatement("UPDATE UZIVATEL SET HESLO = ? WHERE USERNAME = ?");
    }

    @Override
    public void add(Username username, Prijmeni prijmeni, Jmeno jmeno, RoleUzivatele roleUzivatele, Heslo heslo) throws SmpException {
        try {
            createPs.setString(1, username.toString());
            createPs.setString(2, prijmeni.toString());
            createPs.setString(3, jmeno.toString());
            createPs.setString(4, roleUzivatele.getRoleUzivateleId());
            createPs.setString(5, heslo.getHesloMDP());
            createPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(Username username) throws SmpException {
        try {
            deletePs.setString(1, username.getUsername());
            deletePs.executeUpdate();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    Uzivatel loadUzivatel(ResultSet rs) throws SQLException {
        return new Uzivatel(
                new Username(rs.getString(Username.colName())),
                new Prijmeni(rs.getString(Prijmeni.colName())),
                new Jmeno(rs.getString(Jmeno.colName())),
                RoleUzivatele.valueOf(rs.getString(RoleUzivatele.colName())),
                new Heslo(rs.getString(Heslo.colName())));

    }

    @Override
    public Uzivatel find(Username username) throws SmpException {
        try {
            findPs.setString(1, username.getUsername());
            ResultSet rs = findPs.executeQuery();
            if (rs.next()) {
                return loadUzivatel(rs);
            }
            return null;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Uzivatel> all() throws SmpException {
        try {
            List<Uzivatel> list = new ArrayList<>();
            ResultSet rs = findAllPs.executeQuery();
            while (rs.next()) {
                list.add(loadUzivatel(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(Username username, Heslo heslo) throws SmpException {
        try {
            updatePs.setString(1, heslo.getHesloMDP());
            updatePs.setString(2, username.getUsername());
            updatePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }
}
