package pobyty.model;

import java.text.Collator;
import java.util.Locale;
import java.util.Objects;
import javax.validation.ValidationException;

public class Prijmeni implements Comparable<Prijmeni> {

    public static int PRIJMENI_SIZE = 20;
    final Collator c = Collator.getInstance(new Locale("cs"));

    public static String colName() {
        return Prijmeni.class.getSimpleName();
    }

    public static Prijmeni validate(String prijmeni) throws ValidationException {
        if (prijmeni == null) {
            throw new ValidationException("Prosím vyplňte příjmení.");
        }
        prijmeni = prijmeni.trim();
        if (prijmeni.isEmpty()) {
            throw new ValidationException("Prosím vyplňte příjmení.");
        }
        if (prijmeni.length() > PRIJMENI_SIZE) {
            throw new ValidationException(String.format("Prijmeni může být maximálně %d znaků.", PRIJMENI_SIZE));
        }
        return new Prijmeni(prijmeni);
    }
    private String prijmeni;

    public Prijmeni(String prijmeni) {
        this.prijmeni = prijmeni;
    }

    /**
     * @return the prijmeni
     */
    public String getPrijmeni() {
        return prijmeni;
    }

    @Override
    public String toString() {
        return prijmeni;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj.getClass() != getClass()) {
            return false;
        }
        return prijmeni.equals(((Prijmeni) obj).prijmeni);
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 47 * hash + Objects.hashCode(this.prijmeni);
        return hash;
    }

    @Override
    public int compareTo(Prijmeni o) {
        return c.compare(prijmeni, o.getPrijmeni());
    }
}
